# Tkabber presence spy module, version 1.0
# Copyright (C) 2003 Maciek Pasternacki <maciekp@japhy.fnord.org>
#
# Traces presence messages, logging them into tkabber window
# and/or file.
#
# Usage: # source file in ~/.tkabber/config.tcl (outside of postload):
#        source "~/.tkabber/spy.tcl"
#        # If you want to log presence messages to file, set filename:
#        set ::spy::filename "~/.tkabber/logs/presence.log"
#        # Now, either run tkabber from within tkcon and run
#        # ::spy::open_window from there to open spy window,
#        # or bind ::spy::open_window into menu or toolbar,
#        # for example like this:
#        proc menuload {desc} {
#            return [concat $desc \
#                        [list "&Plugins" {} {} 1 \
#                             [list \
#                                  [list command "Spy window" {} {} {} \
#                                       -command { ::spy::open_window } ]]]]
#        }

option add *Spy.timestampforeground Black         widgetDefault
option add *Spy.nickforeground      DarkBlue      widgetDefault
option add *Spy.jidforeground       DarkBlue      widgetDefault
option add *Spy.presenceforeground  DarkRed       widgetDefault
option add *Spy.reasonforeground    DarkMagenta   widgetDefault

namespace eval spy {
    set filename ""
    set the_file ""
}

proc spy::open_window {} {
    global font

    set w .spy
    if {[winfo exists $w]} {
        return
    }

    add_win $w -title [::msgcat::mc "Presence Spy"] \
        -tabtitle [::msgcat::mc "Spy"] \
        -class Spy

    set sw [ScrolledWindow $w.isw -scrollbar vertical]
    pack $sw -side top -fill both -expand yes -in $w

    set log [text $w.log -font $font]
    $sw setwidget $log

    $log configure -state disabled

    $log tag configure timestamp \
	-foreground [option get $w timestampforeground Spy]
    $log tag configure nick \
	-foreground [option get $w nickforeground Spy]
    $log tag configure jid \
	-foreground [option get $w jidforeground Spy]
    $log tag configure presence \
	-foreground [option get $w presenceforeground Spy]
    $log tag configure reason \
	-foreground [option get $w reasonforeground Spy]
}

proc spy::display {jid nick type reason} {
    set w .spy

    if {![winfo exists $w]} return
    set log $w.log

    $log configure -state normal

    $log insert end "[clock format [clock seconds]]" timestamp
    $log insert end " "

    if { "$nick"!="" } {
        $log insert end "$nick" nick
        $log insert end " ("
        $log insert end "$jid" jid
        $log insert end ") "
    } else {
        $log insert end "$jid" jid
        $log insert end " "
    }

    $log insert end "$type" presence

    if { "$reason"!="" } {
        $log insert end " ("
        $log insert end "$reason" reason
        $log insert end ")"
    }
    $log insert end "\n"
    $log see end

    $log configure -state disabled
}

proc spy::client_presence_handler {connid from type x opts} {
    variable the_file

    if {[catch  { set nick [chat::get_nick $from chat] }]} {
        set nick "$from" 
    }

    if { "$nick"=="$from" } {
        set nick ""
    }

    set reason ""
    set show ""
    foreach {attr val} $opts {
	switch -- $attr {
	    -status   {set reason $val}
	    -show     {set show   $val}
	}
    }

    if { "$type"=="" } { set type "online" }
    if { "$show"!="" } { set type "$type/$show" }

    display "$from" "$nick" "$type" "$reason"

    if { "$the_file"!="" } {
        puts -nonewline $the_file "[clock format [clock seconds]] "
        if { "$nick"!="" } {
            puts -nonewline $the_file "$nick ($from) "
        } else {
            puts -nonewline $the_file "$from "
        }
        puts -nonewline $the_file "$type"
        if { "$reason"!="" } {
            puts -nonewline $the_file " ($reason)"
        }
        puts $the_file {}
        flush $the_file
    }
}

proc spy::init_spy {} {
    variable the_file
    variable filename

    if { "$filename"!="" } {
        if { "$the_file"!="" } {
            close $the_file
            set the_file ""
        }
        set the_file [open "$filename" "a"]
        puts $the_file "[clock format [clock seconds]] Started spying."
    }
}

proc spy::deinit_spy {} {
    variable the_file
    if { "$the_file"!="" } {
        puts $the_file "[clock format [clock seconds]] Spy goes away."
        close $the_file
    }
}

proc spy::setup_menu {} {
    catch { 
        set m [.mainframe getmenu roster]

        $m add command -label [::msgcat::mc "Spy presence"] \
	    -command [namespace current]::open_window
    }
}

hook::add postload_hook [namespace current]::spy::init_spy 100
hook::add client_presence_hook [namespace current]::spy::client_presence_handler 100
hook::add quit_hook [namespace current]::spy::deinit_spy 100
hook::add finload_hook [namespace current]::spy::setup_menu
